//  Author : Marveldex Inc. mdex-shop.com
//  Sensor : https://bit.ly/2RlWqkz
//  Source Ver. : 2020-0623a
//  Release : NOT YET
//  Inquiry : sales@mdex.co.kr

import processing.serial.*;
import controlP5.*; // import controlP5 library
ControlP5 controlP5; // controlP5 object


Serial myPort;
SeatDataView seatView;
KeystrokeSimulator keySim;


int       NUM_OF_CELLS = 32;
String[]  stringValues = new String[NUM_OF_CELLS + NUM_OF_CELLS]; // +1 : prevent buffer overflow error.
int []    cellValues = new int[NUM_OF_CELLS];
PImage    imgSensor; // Background image
Table     csvRectPos;

String    strBuffer;
EditRect  textLog;


boolean isDebug = false;

void setup() {
  size(990, 656);
  //colorMode(HSB);
  colorMode(HSB, 360, 100, 100);
  
  //  load background image file
  imgSensor = loadImage("../image/cushion_sensor_680x656.png");
  //imgSensor = loadImage("../image/cell_index.png");
  //imgSensor = loadImage("../image/ivory_cushion.png");
  //imgSensor = loadImage("../image/white_cushion.png");
  //imgSensor = loadImage("../image/base.png");
  image(imgSensor, 0, 0);
  textLog = new EditRect(1, 1, 500, 20);
  
  //  SeatDataView - rect position and color
  seatView = new SeatDataView();

  //  load csv file - rect cell position. <left, top, width, height> of 31 rect
  csvRectPos = loadTable("../image/rect_pos.csv", "header");
  //csvRectPos = loadTable("../image/cell_index.csv", "header");
  //csvRectPos = loadTable("../image/ivory_cushion.csv", "header");
  //csvRectPos = loadTable("../image/white_cushion.csv", "header");
  if(false == seatView.loadCSV_31Rect(csvRectPos)) {
    println("Failed to load CSV file. It should be placed at <../image/rect_pos.csv>");
    return;
  }
  seatView.createCOMGauge();

  //  open serial port
  if(false == openSerialOrExit()) {
    println("Failed to connect with Arduino. Please check connection.");
    return;
  }

  controlP5 = new ControlP5(this);
  PFont p = createFont("Verdana", 15); 
  ControlFont font = new ControlFont(p);
  controlP5.setFont(font);

  // config threshold number box
  controlP5.addNumberbox("up")
    .setValue(-10.0)
    .setPosition(800, 70)
    .setSize(70, 20).setRange(-205, 180)
    .setScrollSensitivity(1.0);

  controlP5.addNumberbox("down")
    .setValue(40.0)
    .setPosition(800, 170)
    .setSize(70, 20).setRange(-205, 180)
    .setScrollSensitivity(1.0);

  controlP5.addNumberbox("left")
    .setValue(-70.0)
    .setPosition(700, 120)
    .setSize(70, 20).setRange(-205, 180)
    .setScrollSensitivity(-1.0);

  controlP5.addNumberbox("right")
    .setValue(70.0)
    .setPosition(900, 120)
    .setSize(70, 20).setRange(-205, 180)
    .setScrollSensitivity(-1.0);

//  keySim.setDirKeyThreshold(-10, 40, -70, 70); // closer to zero, more sensitive. 

  //--------------------------------------------
  keySim = new KeystrokeSimulator();
}

void controlEvent(ControlEvent theEvent) {
  if (theEvent.getController().getName()=="up") {
    println(theEvent.getController().getValue());
  }
}


void draw() {
  if (false == readSerial_MDEX32chShield()) 
    return;


  //  step 1. analyze received buffer.
  seatView.anylizeCellValues(cellValues);

  //  step 2. draw cell values and COM gauge
  seatView.drawColorCells();
  seatView.drawCOMGauge();

  //  step 3. draw the x and y gague of "center of mass"
  PVector   vectorPosture = seatView.getVectorCOM();

  strBuffer = String.format("Center Of Mass  (X, Y) = (%2.4f, %2.4f)", vectorPosture.x, vectorPosture.y);
  drawTextLog(strBuffer);

  //  step 4. do something with analyzed data. For example, make keyboard input.
  keySim.setDirKeyThreshold(-10, 40, -70, 70); // closer to zero, more sensitive. 
  //keySim.simulKeyDirInput(vectorPosture);
  
}


void drawTextLog(String strText) {
  fill(0, 0, 90); // black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
  textLog.render();
  textLog.showString(strText);
}


//  Serial example
//    ==> https://processing.org/reference/libraries/serial/Serial_readStringUntil_.html
boolean openSerialOrExit() {
  if (Serial.list().length == 0) {
    strBuffer = "ERROR ! Can't find serial port. Connect arduino and restart.";
    drawTextLog(strBuffer);
    println(strBuffer);

    delay(1500);
//    exit();
    return false;
  }

  String portName = Serial.list()[0]; // [0] means 0th port in the COM list
  myPort = new Serial(this, portName, 115200); // 115200 : should be same to the setting in Arduino code.

  strBuffer = "Port # is " + portName;
  drawTextLog(strBuffer);
  println(strBuffer);
  delay(500);
  
  return true;
}


int countNull = 0;
boolean readSerial_MDEX32chShield() {
  if ( myPort.available() < 2) { //If receiving buffer is filled under 2 bytes.

    if(myPort.available() == 0) {
      countNull++;      
    }
    if(100 < countNull) 
      drawTextLog("Serial is not connected");

    return false;
  }

  String recvBuf;     // buffer
  recvBuf = myPort.readStringUntil('\n'); // read 1 whole buffer.

  if (recvBuf == null) 
    return false;

  //  check point 1 ) if console shows receiving data
  if (isDebug == true) {
    print("recv)" + recvBuf);
  }

  //  check USB transmission error
  recvBuf = recvBuf.replaceAll(",,", ",");  //  a) Two comma ',,' is an error.
  if (recvBuf.split(",").length < NUM_OF_CELLS ) // b) If the number of data is less than 32... then ignore this buffer. 
    return false;

  //  split string buffer with comma
  stringValues = splitTokens(recvBuf, ",");

  //  convert from string to integer.
  for (int i = 0; i < NUM_OF_CELLS; i++ ) {
    cellValues[i] = Integer.parseInt(stringValues[i]);
  }

  //  check point 2 ) check if parsing is OK. 
  //  fix) If the data is not shown in the console, then change the USB data cable thicker or plug the USB cable to the other port(Power supply problem). 
  if (isDebug == true) {
    print("conv)");
    for (int i = 0; i < NUM_OF_CELLS; i++ ) {
      print(cellValues[i], ",");
    }
    println("end");
  }

  countNull = 0;

  return true;
}
