//////////////////////////////////////////////////////////////////////
//  CLASS  EditRect
//  Rectangle with text
//////////////////////////////////////////////////////////////////////
class EditRect {
  int x, y; // starting point
  int w, h; // width, height
  
  EditRect(int start_x, int start_y, int rect_width, int rect_height){
    x = start_x;
    y = start_y;
    w = rect_width;
    h = rect_height;
  };  
  void setRect(int start_x, int start_y, int rect_width, int rect_height){
    x = start_x;
    y = start_y;
    w = rect_width;
    h = rect_height;
  }
  int getLeft() { return x; };
  int getRight() { return (x + w); };
  int getWidth() { return w; };
  int getTop() { return y; };
  int getBottom() { return (y + h); };
  int getHeight() { return h; };
  int getCenterX() { return (x + w/2);};
  int getCenterY() { return (y + h/2);};

  void render(){
    rect(x, y, w, h);
  }
  
  int text_margin_left = 3;
  int text_shift_down  = 14;
  
  void setTextLeftMargin(int left_margin) {
    text_margin_left = left_margin;
  }
  void shiftTextDown(int shift_size) {
    text_shift_down = shift_size;
  }
  void showInt(int v){
    push();
    fill(0, 0, 0);// black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
    text(v, x + text_margin_left, y + text_shift_down);
    pop();
  }
  void showFloat(float v){
    push();
    fill(0, 0, 0);// black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
    text(v, x + text_margin_left, y + text_shift_down);
    pop();
  }
  void showString(String s){
    push();
    fill(0, 0, 0);// black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
    text(s, x + text_margin_left, y + text_shift_down);
    pop();
  }
}


//////////////////////////////////////////////////////////////////////
//  CLASS  SeatDataView
//  Draw 3 rows of 31 color cells and COM gauge
//////////////////////////////////////////////////////////////////////
class SeatDataView {
  //  num of cells in each row  
  int cellNum_Row0 = 6;
  int cellNum_Row1 = 15;
  int cellNum_Row2 = 10;
  int cellNum_Total = 31; // Row0 + Row1 + Row2
  
  
  int OCCUPY_THRESHOLD = 50;
  
  
  int [] seatValues_Row0 = new int [cellNum_Row0];
  int [] seatValues_Row1 = new int [cellNum_Row1];
  int [] seatValues_Row2 = new int [cellNum_Row2];
  
  EditRect [] EditRect_Row0 = new EditRect [cellNum_Row0];
  EditRect [] EditRect_Row1 = new EditRect [cellNum_Row1];
  EditRect [] EditRect_Row2 = new EditRect [cellNum_Row2];
  
  EditRect rectOutline; // outline rectangular of 31 cells
  
  EditRect RectCOM_X;
  EditRect RectCOM_Y;
  

  PVector vectCOM;
  PVector coordCOM; // UI x, y coordinate of COM
  
  int VALID_VALUE = 5;

  SeatDataView() {
    vectCOM = new PVector(0.0f, 0.0f);
    coordCOM = new PVector(0.0f, 0.0f);

    //makeMy31Rect();
    //createCOMGauge();
  };


  boolean loadCSV_31Rect(Table table) {
    //Table table = file_name;
    //table = loadTable("../image/rect_pos.csv", "header");
    
    if(table.getRowCount() != cellNum_Total)
      return false;
      
    println(table.getRowCount() + " total rows in rect_pos.csv file");
    
    //  load csv file
    String cell_name;
    int [] rect_left  = new int [cellNum_Total];
    int [] rect_top   = new int [cellNum_Total];
    int [] rect_width = new int [cellNum_Total];
    int [] rect_height = new int [cellNum_Total];

    int i = 0;
    for (TableRow row : table.rows()) {
      cell_name       = row.getString("[Row-Col]");
      rect_left[i]    = row.getInt("left");
      rect_top[i]     = row.getInt("top");
      rect_width[i]   = row.getInt("width");
      rect_height[i]  = row.getInt("height");
       
      //println(cell_name + ", " + rect_left[i] + ", " + rect_top[i] + ", " + rect_width[i] + ", " + rect_height[i]);
      i++;
    }
    
    
    //  create Rect UI
    i = 0;
    int col_index = 0; // column index
    //  Row 0
    for (col_index = 0; col_index < cellNum_Row0; col_index++) {
      EditRect_Row0[col_index] = new EditRect(rect_left[i], rect_top[i], rect_width[i], rect_height[i]);
      i++;
    }
    
    //  Row 1
    for (col_index = 0; col_index < cellNum_Row1; col_index++) {
      EditRect_Row1[col_index] = new EditRect(rect_left[i], rect_top[i], rect_width[i], rect_height[i]);
      i++;
    }
    
    //  Row 2
    for (col_index = 0; col_index < cellNum_Row2; col_index++) {
      EditRect_Row2[col_index] = new EditRect(rect_left[i], rect_top[i], rect_width[i], rect_height[i]);
      i++;
    } 
    
    learnGeometry();
    
    return true;
  }
    
  void makeMy31Rect() {
    int rect_margin_top = 68;
    int rect_margin_left = 70;
    int rectWidthWide = 50;
    int rectWidth = 35;
    int rectHeight = 20;
    int intervalLateral = 2;
    int intervalVertical = 195;
  
    int i = 0;
    
  
    for (i = 0; i < cellNum_Row0; i++) {
      if (i < 3) 
        EditRect_Row0[i] = new EditRect(50 + rect_margin_left+(i* (rectWidthWide + intervalLateral + 0)), rect_margin_top + 5, rectWidthWide, rectHeight);
      else
        EditRect_Row0[i] = new EditRect(185 + rect_margin_left+(i* (rectWidthWide + intervalLateral + 0)), rect_margin_top + 5, rectWidthWide, rectHeight);
    }
  
    for (i = 0; i < cellNum_Row1; i++) {
      EditRect_Row1[i] = new EditRect(rect_margin_left+(i* (rectWidth + intervalLateral)), 15 + rect_margin_top + 1 * intervalVertical, rectWidth, rectHeight);
    }
  
    for (i = 0; i < cellNum_Row2; i++) {
      if (i < 5)
        EditRect_Row2[i] = new EditRect(30+ rect_margin_left+(i* (rectWidth + intervalLateral)), rect_margin_top + 2 * intervalVertical, rectWidth, rectHeight);
      else
        EditRect_Row2[i] = new EditRect(145+ rect_margin_left+(i* (rectWidth + intervalLateral)), rect_margin_top + 2 * intervalVertical, rectWidth, rectHeight);
    }  
    
    learnGeometry();
  }
  

  void learnGeometry(){
    EditRect rectThis = EditRect_Row0[0];
    
    int left_end, right_end, top_end, bottom_end;

    left_end = right_end = rectThis.getCenterX();
    top_end = bottom_end = rectThis.getCenterY();
    
    int rect_x, rect_y;
    
    int col_index = 0; // column index
    //  find outline of 31 cells
    //  Row 0
    for (col_index = 0; col_index < cellNum_Row0; col_index++) {
      rect_x = EditRect_Row0[col_index].getCenterX();
      rect_y = EditRect_Row0[col_index].getCenterY();

      if(rect_x < left_end) 
        left_end = rect_x;
      if(right_end < rect_x)
        right_end = rect_x;

      if(rect_y < top_end)
        top_end = rect_y;
      if(bottom_end < rect_y)
        bottom_end = rect_y;
    }
    
    //  Row 1
    for (col_index = 0; col_index < cellNum_Row1; col_index++) {
      rect_x = EditRect_Row1[col_index].getCenterX();
      rect_y = EditRect_Row1[col_index].getCenterY();

      if(rect_x < left_end)
        left_end = rect_x;
      if(right_end < rect_x)
        right_end = rect_x;

      if(rect_y < top_end)
        top_end = rect_y;
      if(bottom_end < rect_y)
        bottom_end = rect_y;
    }
    
    //  Row 2
    for (col_index = 0; col_index < cellNum_Row2; col_index++) {
      rect_x = EditRect_Row2[col_index].getCenterX();
      rect_y = EditRect_Row2[col_index].getCenterY();

      if(rect_x < left_end)
        left_end = rect_x;
      if(right_end < rect_x)
        right_end = rect_x;

      if(rect_y < top_end)
        top_end = rect_y;
      if(bottom_end < rect_y)
        bottom_end = rect_y;
    } 

    //  calculate outline, center
    rectOutline = new EditRect(left_end, top_end, (right_end - left_end), (bottom_end - top_end)); // new EditRect(left, top, width, height); 

    println("outline(l,r,t,b) : " + left_end + ", " + right_end + ", " + top_end + ", " + bottom_end);
    
  }


  
  void createCOMGauge() {
    int gaugeX_height = 20;
    RectCOM_X = new EditRect(rectOutline.getLeft(), EditRect_Row1[0].getBottom(), rectOutline.getWidth(), gaugeX_height ); // left, top, width, height
    RectCOM_X.setTextLeftMargin(rectOutline.getWidth() / 2);
    
    int gaugeY_width = 40;
    RectCOM_Y = new EditRect(4, rectOutline.getTop(), gaugeY_width, rectOutline.getHeight()); // left, top, width, height
    RectCOM_Y.setTextLeftMargin(0);
    RectCOM_Y.shiftTextDown(rectOutline.getHeight() / 3);
  }
  
  // color map of HSB
  int colorMap(int cell_value) {
    int hue = 180 - (cell_value * 180 / 220 ); // make 0 ~ 220(cell) to 180 ~ 0(hue)

    if(240 < cell_value)
      hue = 0;

    return hue;
  }
  
  void drawColorCells(){  
    int i = 0;
  
    for (i = 0; i < cellNum_Row0; i++) {
      fill(colorMap(seatValues_Row0[i]), 100, 100); // black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
      EditRect_Row0[i].render();  
      EditRect_Row0[i].showInt(seatValues_Row0[i]);
    } 
  
    for (i = 0; i < cellNum_Row1; i++) {
      fill(colorMap(seatValues_Row1[i]), 100, 100); // black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
      EditRect_Row1[i].render();  
      EditRect_Row1[i].showInt(seatValues_Row1[i]);
    } 
  
    for (i = 0; i < cellNum_Row2; i++) {
      fill(colorMap(seatValues_Row2[i]), 100, 100); // black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
      EditRect_Row2[i].render();  
      EditRect_Row2[i].showInt(seatValues_Row2[i]);
    } 
  }
  
  void drawCOMGauge() {
    push();
    //  draw X gauge
    fill(0, 0, 90); // black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
    RectCOM_X.render();
    RectCOM_X.showFloat(vectCOM.x);
  
     // [0] : Any index of Row1(0~14) can replace 0.
    fill(0, 0, 0); // black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
    line(coordCOM.x, EditRect_Row1[0].getBottom(), coordCOM.x, EditRect_Row1[0].getBottom() + 20);
  
    //  draw Y gauge
    fill(0, 0, 90); // black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
    RectCOM_Y.render();
    RectCOM_Y.showFloat(vectCOM.y);
    //  4, 30 : position coordinates
    stroke(0, 50, 50);
    //fill(360, 0, 0); // black(0, 0, 0) . white(0, 0, 100), color (180~0, 100, 100) 
    line(14, coordCOM.y, 14 + 30, coordCOM.y);
    pop();
  }


  void anylizeCellValues(int [] cell_values) {
    reorderPacketToSeatView(cell_values);
    analyzeDist();
  }

  void reorderPacketToSeatView(int [] packet_data) {
    //  see the image 'seat sensor cell index.png'
    seatValues_Row0[0] = packet_data[10];
    seatValues_Row0[1] = packet_data[12];
    seatValues_Row0[2] = packet_data[14];
    seatValues_Row0[3] = packet_data[16];
    seatValues_Row0[4] = packet_data[18];
    seatValues_Row0[5] = packet_data[20];
  
    seatValues_Row1[0] = packet_data[5];
    seatValues_Row1[1] = packet_data[6];
    seatValues_Row1[2] = packet_data[7];  
    seatValues_Row1[3] = packet_data[8];
    seatValues_Row1[4] = packet_data[9];
    seatValues_Row1[5] = packet_data[11];
    seatValues_Row1[6] = packet_data[13];
    seatValues_Row1[7] = packet_data[15];
    seatValues_Row1[8] = packet_data[17];
    seatValues_Row1[9] = packet_data[19];
    seatValues_Row1[10] = packet_data[21];
    seatValues_Row1[11] = packet_data[22];
    seatValues_Row1[12] = packet_data[23];
    seatValues_Row1[13] = packet_data[24];
    seatValues_Row1[14] = packet_data[25];
  
    seatValues_Row2[0] = packet_data[0];
    seatValues_Row2[1] = packet_data[1];
    seatValues_Row2[2] = packet_data[2];
    seatValues_Row2[3] = packet_data[3];
    seatValues_Row2[4] = packet_data[4];
    seatValues_Row2[5] = packet_data[26];
    seatValues_Row2[6] = packet_data[27];
    seatValues_Row2[7] = packet_data[28];
    seatValues_Row2[8] = packet_data[29];
    seatValues_Row2[9] = packet_data[30];
  }


  //  anylyze distribution such as center of mass, ...
  void analyzeDist() {    
    //  calc COM. COM : Center of Mass
    vectCOM.x = calcCOM_X();    
    vectCOM.y = calcCOM_Y();

    //  calc coordinates of COM
    coordCOM.x = rectOutline.getCenterX() + vectCOM.x * rectOutline.getWidth() / 2;
    coordCOM.y = rectOutline.getCenterY() - vectCOM.y * rectOutline.getHeight() / 2;
    
    println("vx=" + coordCOM.x + ", vy=" + coordCOM.y);
  }

  PVector getVectorCOM() {
    //  trimming. 0.12345678 to 0.1234
    int x10000 = int(vectCOM.x * 10000.0f);
    vectCOM.x = float(x10000) / 10000;
    
    int y10000 = int(vectCOM.y * 10000.0f);
    vectCOM.y = float(y10000) / 10000;

    return vectCOM;
  }


  //  calculate center of mass of X
  //  refer to : http://www.softschools.com/formulas/physics/center_of_mass_formula/87/
  //  consider only row 1 of 3 rows to calculate COM x
  float calcCOM_X() {
    float com_x = 0.0f;
    float sum_weight_pos = 0.0f;
    int sum_weight=0;
  
    float pitch_x = 2.0f / (cellNum_Row1 - 1) ; // 2/(15-1). 2 means the length of -1~1. (15-1) means the number of interval of 15 cells. 
    
    
    for (int i = 0; i < cellNum_Row1; i++) {
      float cell_pos_x = -1.0f + i * pitch_x; // -1.0f means the most left cell's x-position while the most right cell is 1.0f.
      sum_weight_pos += seatValues_Row1[i] * cell_pos_x;
      sum_weight += seatValues_Row1[i];
    }
  
    if ( OCCUPY_THRESHOLD < sum_weight ) {
      com_x = sum_weight_pos / sum_weight;
    } else {
      com_x = 0.0f; // 0.0f : center Row1(-1.0f ~ 1.0f)
    }
    //println("com_x : " + com_x);

    return com_x;
  }


  //  calculate center of mass of Y
  //  refer to : http://www.softschools.com/formulas/physics/center_of_mass_formula/87/
  float calcCOM_Y() {
    float com_y = 0.0f;
    float sum_weight_pos = 0.0f;
    int sum_weight=0;
  
    for (int i = 0; i < cellNum_Row0; i++) { 
      sum_weight_pos += seatValues_Row0[i] * 1.0f; // Y position of Row0 = 1
      sum_weight += seatValues_Row0[i];
    }
    for (int i = 0; i < cellNum_Row1; i++) { 
      sum_weight_pos += seatValues_Row1[i] * 0.0f; // Y position of Row1 = 0
      sum_weight += seatValues_Row1[i];
    }
    for (int i = 0; i < cellNum_Row2; i++) { 
      sum_weight_pos += seatValues_Row2[i] * (-1.0f); // Y position of Row2 = -1
      sum_weight += seatValues_Row2[i];
    }
  
    if (OCCUPY_THRESHOLD < sum_weight) {
      com_y = sum_weight_pos / sum_weight;
    } else {
      com_y = 0.0f;
    }
  
    //println("com_y : " + com_y);

    return com_y;
  }
  
  //  Calculate COC. COC : Center of Contour
  int iLeftEnd, iRightEnd;
  float coordCOC_X, coordLeftEnd, coordRightEnd;
  float coordRCOM_X;

  int INVALID_INDEX = (-1);
  
  void calcCOC() {
    iLeftEnd = iRightEnd = INVALID_INDEX;
    coordCOC_X = coordLeftEnd = coordRightEnd = EditRect_Row2[7].getCenterX();

    for(int i = 0 ; i < cellNum_Row1 ; i++) {
      if(VALID_VALUE < seatValues_Row1[i]) {
        iLeftEnd = i;
        break;
      }      
    }
    for(int i = ( - 1) ; 0 <= i ; i--) {
      if(VALID_VALUE < seatValues_Row1[i]) {
        iRightEnd = i;
        break;
      }
    }
    if( (iLeftEnd == INVALID_INDEX) || (iRightEnd == INVALID_INDEX) ) {
      coordCOC_X = INVALID_INDEX;
    }
    else {
      coordLeftEnd = EditRect_Row2[iLeftEnd].getCenterX();
      coordRightEnd = EditRect_Row2[iRightEnd].getCenterX();
      coordCOC_X = ( coordLeftEnd + coordRightEnd ) / 2;
    }
        
    //  calc RCOM
    coordRCOM_X = coordCOM.x - coordCOC_X;
  }

  
  
    
/*  
  void simulKeyDirInput() {
    //  key simulation
    if ( 70 < (coordCOM.x - EditRect_Row1[7].getCenterX()) ) { // [7] : center index of Row1(0~14)
      keySim.simKeyEvent(KeyEvent.VK_RIGHT);
    } else if ( (coordCOM.x - EditRect_Row1[7].getCenterX()) < -70 ) { // [7] : center index of Row1(0~14)
      keySim.simKeyEvent(KeyEvent.VK_LEFT);
    }
  
    if ( 70 < (coordCOM.y - EditRect_Row1[0].getCenterY()) ) { // [0] : Any index of Row1(0~14) can replace 0.
      keySim.simKeyEvent(KeyEvent.VK_DOWN);
    } else if ( (coordCOM.y - EditRect_Row1[0].getCenterY()) < -70 ) { // [0] : Any index of Row1(0~14) can replace 0.
      keySim.simKeyEvent(KeyEvent.VK_UP);
    }
  }
*/  
  
} // End of SeatDataView class
